@@ -83,16 +83,21 @@ bool AST::isInterface() const {
     return mRootScope.getInterface() != nullptr;
 }
 
-bool AST::containsInterfaces() const {
-    return mRootScope.containsInterfaces();
+bool AST::definesInterfaces() const {
+    return mRootScope.definesInterfaces();
 }
 
 status_t AST::postParse() {
     status_t err;
 
-    // lookupTypes is the first pass.
+    // lookupTypes is the first pass for references to be resolved.
     err = lookupTypes();
     if (err != OK) return err;
+
+    // Indicate that all types are now in "postParse" stage.
+    err = setParseStage(Type::ParseStage::PARSE, Type::ParseStage::POST_PARSE);
+    if (err != OK) return err;
+
     // validateDefinedTypesUniqueNames is the first call
     // after lookup, as other errors could appear because
     // user meant different type than we assumed.
@@ -104,13 +109,15 @@ status_t AST::postParse() {
     if (err != OK) return err;
     err = resolveInheritance();
     if (err != OK) return err;
-    err = lookupLocalIdentifiers();
+    err = lookupConstantExpressions();
     if (err != OK) return err;
     // checkAcyclicConstantExpressions is after resolveInheritance,
     // as resolveInheritance autofills enum values.
     err = checkAcyclicConstantExpressions();
     if (err != OK) return err;
-    err = evaluate();
+    err = validateConstantExpressions();
+    if (err != OK) return err;
+    err = evaluateConstantExpressions();
     if (err != OK) return err;
     err = validate();
     if (err != OK) return err;
@@ -127,13 +134,9 @@ status_t AST::postParse() {
             return OK;
         },
         true /* processBeforeDependencies */);
-    std::unordered_set<const Type*> visited;
-    mRootScope.recursivePass(
-        [](Type* type) {
-            type->setPostParseCompleted();
-            return OK;
-        },
-        &visited);
+
+    err = setParseStage(Type::ParseStage::POST_PARSE, Type::ParseStage::COMPLETED);
+    if (err != OK) return err;
 
     return OK;
 }
@@ -142,20 +145,50 @@ status_t AST::constantExpressionRecursivePass(
     const std::function<status_t(ConstantExpression*)>& func, bool processBeforeDependencies) {
     std::unordered_set<const Type*> visitedTypes;
     std::unordered_set<const ConstantExpression*> visitedCE;
-    return mRootScope.recursivePass(
-        [&](Type* type) -> status_t {
-            for (auto* ce : type->getConstantExpressions()) {
-                status_t err = ce->recursivePass(func, &visitedCE, processBeforeDependencies);
-                if (err != OK) return err;
-            }
-            return OK;
-        },
-        &visitedTypes);
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE,
+                                    [&](Type* type) -> status_t {
+                                        for (auto* ce : type->getConstantExpressions()) {
+                                            status_t err = ce->recursivePass(
+                                                func, &visitedCE, processBeforeDependencies);
+                                            if (err != OK) return err;
+                                        }
+                                        return OK;
+                                    },
+                                    &visitedTypes);
+}
+
+status_t AST::constantExpressionRecursivePass(
+    const std::function<status_t(const ConstantExpression*)>& func,
+    bool processBeforeDependencies) const {
+    std::unordered_set<const Type*> visitedTypes;
+    std::unordered_set<const ConstantExpression*> visitedCE;
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE,
+                                    [&](const Type* type) -> status_t {
+                                        for (auto* ce : type->getConstantExpressions()) {
+                                            status_t err = ce->recursivePass(
+                                                func, &visitedCE, processBeforeDependencies);
+                                            if (err != OK) return err;
+                                        }
+                                        return OK;
+                                    },
+                                    &visitedTypes);
+}
+
+status_t AST::setParseStage(Type::ParseStage oldStage, Type::ParseStage newStage) {
+    std::unordered_set<const Type*> visited;
+    return mRootScope.recursivePass(oldStage,
+                                    [oldStage, newStage](Type* type) {
+                                        CHECK(type->getParseStage() == oldStage);
+                                        type->setParseStage(newStage);
+                                        return OK;
+                                    },
+                                    &visited);
 }
 
 status_t AST::lookupTypes() {
     std::unordered_set<const Type*> visited;
     return mRootScope.recursivePass(
+        Type::ParseStage::PARSE,
         [&](Type* type) -> status_t {
             Scope* scope = type->isScope() ? static_cast<Scope*>(type) : type->parent();
 
@@ -182,6 +215,7 @@ status_t AST::lookupTypes() {
 status_t AST::gatherReferencedTypes() {
     std::unordered_set<const Type*> visited;
     return mRootScope.recursivePass(
+        Type::ParseStage::POST_PARSE,
         [&](Type* type) -> status_t {
             for (auto* nextRef : type->getReferences()) {
                 const Type *targetType = nextRef->get();
@@ -196,11 +230,12 @@ status_t AST::gatherReferencedTypes() {
         &visited);
 }
 
-status_t AST::lookupLocalIdentifiers() {
+status_t AST::lookupConstantExpressions() {
     std::unordered_set<const Type*> visitedTypes;
     std::unordered_set<const ConstantExpression*> visitedCE;
 
     return mRootScope.recursivePass(
+        Type::ParseStage::POST_PARSE,
         [&](Type* type) -> status_t {
             Scope* scope = type->isScope() ? static_cast<Scope*>(type) : type->parent();
 
@@ -214,6 +249,18 @@ status_t AST::lookupLocalIdentifiers() {
                             if (iden == nullptr) return UNKNOWN_ERROR;
                             nextRef->set(iden);
                         }
+                        for (auto* nextRef : ce->getTypeReferences()) {
+                            if (nextRef->isResolved()) continue;
+
+                            Type* nextType = lookupType(nextRef->getLookupFqName(), scope);
+                            if (nextType == nullptr) {
+                                std::cerr << "ERROR: Failed to lookup type '"
+                                          << nextRef->getLookupFqName().string() << "' at "
+                                          << nextRef->location() << "\n";
+                                return UNKNOWN_ERROR;
+                            }
+                            nextRef->set(nextType);
+                        }
                         return OK;
                     },
                     &visitedCE, true /* processBeforeDependencies */);
@@ -228,6 +275,7 @@ status_t AST::lookupLocalIdentifiers() {
 status_t AST::validateDefinedTypesUniqueNames() const {
     std::unordered_set<const Type*> visited;
     return mRootScope.recursivePass(
+        Type::ParseStage::POST_PARSE,
         [&](const Type* type) -> status_t {
             // We only want to validate type definition names in this place.
             if (type->isScope()) {
@@ -240,10 +288,17 @@ status_t AST::validateDefinedTypesUniqueNames() const {
 
 status_t AST::resolveInheritance() {
     std::unordered_set<const Type*> visited;
-    return mRootScope.recursivePass(&Type::resolveInheritance, &visited);
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE, &Type::resolveInheritance,
+                                    &visited);
 }
 
-status_t AST::evaluate() {
+status_t AST::validateConstantExpressions() const {
+    return constantExpressionRecursivePass(
+        [](const ConstantExpression* ce) { return ce->validate(); },
+        true /* processBeforeDependencies */);
+}
+
+status_t AST::evaluateConstantExpressions() {
     return constantExpressionRecursivePass(
         [](ConstantExpression* ce) {
             ce->evaluate();
@@ -254,7 +309,7 @@ status_t AST::evaluate() {
 
 status_t AST::validate() const {
     std::unordered_set<const Type*> visited;
-    return mRootScope.recursivePass(&Type::validate, &visited);
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE, &Type::validate, &visited);
 }
 
 status_t AST::topologicalReorder() {
@@ -264,14 +319,14 @@ status_t AST::topologicalReorder() {
     if (err != OK) return err;
 
     std::unordered_set<const Type*> visited;
-    mRootScope.recursivePass(
-        [&](Type* type) {
-            if (type->isScope()) {
-                static_cast<Scope*>(type)->topologicalReorder(reversedOrder);
-            }
-            return OK;
-        },
-        &visited);
+    mRootScope.recursivePass(Type::ParseStage::POST_PARSE,
+                             [&](Type* type) {
+                                 if (type->isScope()) {
+                                     static_cast<Scope*>(type)->topologicalReorder(reversedOrder);
+                                 }
+                                 return OK;
+                             },
+                             &visited);
     return OK;
 }
 
@@ -279,29 +334,31 @@ status_t AST::checkAcyclicConstantExpressions() const {
     std::unordered_set<const Type*> visitedTypes;
     std::unordered_set<const ConstantExpression*> visitedCE;
     std::unordered_set<const ConstantExpression*> stack;
-    return mRootScope.recursivePass(
-        [&](const Type* type) -> status_t {
-            for (auto* ce : type->getConstantExpressions()) {
-                status_t err = ce->checkAcyclic(&visitedCE, &stack).status;
-                CHECK(err != OK || stack.empty());
-                if (err != OK) return err;
-            }
-            return OK;
-        },
-        &visitedTypes);
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE,
+                                    [&](const Type* type) -> status_t {
+                                        for (auto* ce : type->getConstantExpressions()) {
+                                            status_t err =
+                                                ce->checkAcyclic(&visitedCE, &stack).status;
+                                            CHECK(err != OK || stack.empty());
+                                            if (err != OK) return err;
+                                        }
+                                        return OK;
+                                    },
+                                    &visitedTypes);
 }
 
 status_t AST::checkForwardReferenceRestrictions() const {
     std::unordered_set<const Type*> visited;
-    return mRootScope.recursivePass(
-        [](const Type* type) -> status_t {
-            for (const Reference<Type>* ref : type->getReferences()) {
-                status_t err = type->checkForwardReferenceRestrictions(*ref);
-                if (err != OK) return err;
-            }
-            return OK;
-        },
-        &visited);
+    return mRootScope.recursivePass(Type::ParseStage::POST_PARSE,
+                                    [](const Type* type) -> status_t {
+                                        for (const Reference<Type>* ref : type->getReferences()) {
+                                            status_t err =
+                                                type->checkForwardReferenceRestrictions(*ref);
+                                            if (err != OK) return err;
+                                        }
+                                        return OK;
+                                    },
+                                    &visited);
 }
 
 bool AST::addImport(const char *import) {
@@ -448,7 +505,6 @@ EnumValue* AST::lookupEnumValue(const FQName& fqName, std::string* errorMsg, Sco
     FQName enumTypeName = fqName.typeName();
     std::string enumValueName = fqName.valueName();
 
-    CHECK(enumTypeName.isValid());
     CHECK(!enumValueName.empty());
 
     Type* type = lookupType(enumTypeName, scope);
@@ -475,8 +531,6 @@ EnumValue* AST::lookupEnumValue(const FQName& fqName, std::string* errorMsg, Sco
 }
 
 Type* AST::lookupType(const FQName& fqName, Scope* scope) {
-    CHECK(fqName.isValid());
-
     if (fqName.name().empty()) {
         // Given a package and version???
         return nullptr;
