@@ -103,17 +103,43 @@ void AST::enterLeaveNamespace(Formatter &out, bool enter) const {
 static void declareGetService(Formatter &out, const std::string &interfaceName, bool isTry) {
     const std::string functionName = isTry ? "tryGetService" : "getService";
 
+    if (isTry) {
+        DocComment(
+                "This gets the service of this type with the specified instance name. If the\n"
+                "service is currently not available or not in the VINTF manifest on a Trebilized\n"
+                "device, this will return nullptr. This is useful when you don't want to block\n"
+                "during device boot. If getStub is true, this will try to return an unwrapped\n"
+                "passthrough implementation in the same process. This is useful when getting an\n"
+                "implementation from the same partition/compilation group.\n\n"
+                "In general, prefer getService(std::string,bool)")
+                .emit(out);
+    } else {
+        DocComment(
+                "This gets the service of this type with the specified instance name. If the\n"
+                "service is not in the VINTF manifest on a Trebilized device, this will return\n"
+                "nullptr. If the service is not available, this will wait for the service to\n"
+                "become available. If the service is a lazy service, this will start the service\n"
+                "and return when it becomes available. If getStub is true, this will try to\n"
+                "return an unwrapped passthrough implementation in the same process. This is\n"
+                "useful when getting an implementation from the same partition/compilation group.")
+                .emit(out);
+    }
     out << "static ::android::sp<" << interfaceName << "> " << functionName << "("
         << "const std::string &serviceName=\"default\", bool getStub=false);\n";
+    DocComment("Deprecated. See " + functionName + "(std::string, bool)").emit(out);
     out << "static ::android::sp<" << interfaceName << "> " << functionName << "("
         << "const char serviceName[], bool getStub=false)"
         << "  { std::string str(serviceName ? serviceName : \"\");"
         << "      return " << functionName << "(str, getStub); }\n";
+    DocComment("Deprecated. See " + functionName + "(std::string, bool)").emit(out);
     out << "static ::android::sp<" << interfaceName << "> " << functionName << "("
         << "const ::android::hardware::hidl_string& serviceName, bool getStub=false)"
         // without c_str the std::string constructor is ambiguous
         << "  { std::string str(serviceName.c_str());"
         << "      return " << functionName << "(str, getStub); }\n";
+    DocComment("Calls " + functionName +
+               "(\"default\", bool). This is the recommended instance name for singleton services.")
+            .emit(out);
     out << "static ::android::sp<" << interfaceName << "> " << functionName << "("
         << "bool getStub) { return " << functionName << "(\"default\", getStub); }\n";
 }
@@ -122,8 +148,13 @@ static void declareServiceManagerInteractions(Formatter &out, const std::string
     declareGetService(out, interfaceName, true /* isTry */);
     declareGetService(out, interfaceName, false /* isTry */);
 
+    DocComment(
+            "Registers a service with the service manager. For Trebilized devices, the service\n"
+            "must also be in the VINTF manifest.")
+            .emit(out);
     out << "__attribute__ ((warn_unused_result))"
         << "::android::status_t registerAsService(const std::string &serviceName=\"default\");\n";
+    DocComment("Registers for notifications for when a service is registered.").emit(out);
     out << "static bool registerForNotifications(\n";
     out.indent(2, [&] {
         out << "const std::string &serviceName,\n"
@@ -140,8 +171,7 @@ static void implementGetService(Formatter &out,
     const std::string interfaceName = fqName.getInterfaceName();
     const std::string functionName = isTry ? "tryGetService" : "getService";
 
-    out << "// static\n"
-        << "::android::sp<" << interfaceName << "> " << interfaceName << "::" << functionName << "("
+    out << "::android::sp<" << interfaceName << "> " << interfaceName << "::" << functionName << "("
         << "const std::string &serviceName, const bool getStub) ";
     out.block([&] {
         out << "return ::android::hardware::details::getServiceInternal<"
@@ -163,20 +193,7 @@ static void implementServiceManagerInteractions(Formatter &out,
     out << "::android::status_t " << interfaceName << "::registerAsService("
         << "const std::string &serviceName) ";
     out.block([&] {
-        out << "::android::hardware::details::onRegistration(\""
-            << fqName.getPackageAndVersion().string() << "\", \""
-            << interfaceName
-            << "\", serviceName);\n\n";
-        out << "const ::android::sp<::android::hidl::manager::V1_0::IServiceManager> sm\n";
-        out.indent(2, [&] {
-            out << "= ::android::hardware::defaultServiceManager();\n";
-        });
-        out.sIf("sm == nullptr", [&] {
-            out << "return ::android::INVALID_OPERATION;\n";
-        }).endl();
-        out << "::android::hardware::Return<bool> ret = "
-            << "sm->add(serviceName.c_str(), this);\n"
-            << "return ret.isOk() && ret ? ::android::OK : ::android::UNKNOWN_ERROR;\n";
+        out << "return ::android::hardware::details::registerAsServiceInternal(this, serviceName);\n";
     }).endl().endl();
 
     out << "bool " << interfaceName << "::registerForNotifications(\n";
@@ -242,12 +259,14 @@ void AST::generateInterfaceHeader(Formatter& out) const {
     out << "\n";
 
     if (iface) {
+        iface->emitDocComment(out);
+
         out << "struct "
             << ifaceName;
 
         const Interface *superType = iface->superType();
 
-        if (superType == NULL) {
+        if (superType == nullptr) {
             out << " : virtual public ::android::RefBase";
         } else {
             out << " : public "
@@ -258,17 +277,28 @@ void AST::generateInterfaceHeader(Formatter& out) const {
 
         out.indent();
 
+        DocComment("Type tag for use in template logic that indicates this is a 'pure' class.")
+                .emit(out);
         generateCppTag(out, "android::hardware::details::i_tag");
+
+        DocComment("Fully qualified interface name: \"" + iface->fqName().string() + "\"")
+                .emit(out);
+        out << "static const char* descriptor;\n\n";
+
+        iface->emitTypeDeclarations(out);
+    } else {
+        mRootScope.emitTypeDeclarations(out);
     }
 
-    emitTypeDeclarations(out);
-
     if (iface) {
+        DocComment(
+                "Returns whether this object's implementation is outside of the current process.")
+                .emit(out);
         out << "virtual bool isRemote() const ";
         if (!isIBase()) {
             out << "override ";
         }
-        out << "{ return false; }\n\n";
+        out << "{ return false; }\n";
 
         for (const auto& tuple : iface->allMethodsFromRoot()) {
             const Method* method = tuple.method();
@@ -279,6 +309,7 @@ void AST::generateInterfaceHeader(Formatter& out) const {
             const NamedReference<Type>* elidedReturn = method->canElideCallback();
 
             if (elidedReturn == nullptr && returnsValue) {
+                DocComment("Return callback for " + method->name()).emit(out);
                 out << "using "
                     << method->name()
                     << "_cb = std::function<void(";
@@ -311,10 +342,14 @@ void AST::generateInterfaceHeader(Formatter& out) const {
             out << ";\n";
         }
 
-        out << "// cast static functions\n";
+        out << "\n// cast static functions\n";
         std::string childTypeResult = iface->getCppResultType();
 
         for (const Interface *superType : iface->typeChain()) {
+            DocComment(
+                    "This performs a checked cast based on what the underlying implementation "
+                    "actually is.")
+                    .emit(out);
             out << "static ::android::hardware::Return<"
                 << childTypeResult
                 << "> castFrom("
@@ -323,11 +358,10 @@ void AST::generateInterfaceHeader(Formatter& out) const {
                 << ", bool emitError = false);\n";
         }
 
-        out << "\nstatic const char* descriptor;\n\n";
-
         if (isIBase()) {
-            out << "// skipped getService, registerAsService, registerForNotifications\n\n";
+            out << "\n// skipped getService, registerAsService, registerForNotifications\n\n";
         } else {
+            out << "\n// helper methods for interactions with the hwservicemanager\n";
             declareServiceManagerInteractions(out, iface->localName());
         }
     }
@@ -338,11 +372,22 @@ void AST::generateInterfaceHeader(Formatter& out) const {
         out << "};\n\n";
     }
 
+    out << "//\n";
+    out << "// type declarations for package\n";
+    out << "//\n\n";
     mRootScope.emitPackageTypeDeclarations(out);
+    out << "//\n";
+    out << "// type header definitions for package\n";
+    out << "//\n\n";
+    mRootScope.emitPackageTypeHeaderDefinitions(out);
 
     out << "\n";
     enterLeaveNamespace(out, false /* enter */);
+    out << "\n";
 
+    out << "//\n";
+    out << "// global type declarations for package\n";
+    out << "//\n\n";
     mRootScope.emitGlobalTypeDeclarations(out);
 
     out << "\n#endif  // " << guard << "\n";
@@ -387,18 +432,12 @@ void AST::generateHwBinderHeader(Formatter& out) const {
     out << "\n#endif  // " << guard << "\n";
 }
 
-void AST::emitTypeDeclarations(Formatter& out) const {
-    return mRootScope.emitTypeDeclarations(out);
-}
-
-static void wrapPassthroughArg(Formatter& out, const NamedReference<Type>* arg,
-                               bool addPrefixToName, std::function<void(void)> handleError) {
+static std::string wrapPassthroughArg(Formatter& out, const NamedReference<Type>* arg,
+                                      std::string name, std::function<void(void)> handleError) {
     if (!arg->type().isInterface()) {
-        return;
+        return name;
     }
-    std::string name = (addPrefixToName ? "_hidl_out_" : "") + arg->name();
-    std::string wrappedName = (addPrefixToName ? "_hidl_out_wrapped_" : "_hidl_wrapped_")
-            + arg->name();
+    std::string wrappedName = "_hidl_wrapped_" + name;
     const Interface &iface = static_cast<const Interface &>(arg->type());
     out << iface.getCppStackType() << " " << wrappedName << ";\n";
     // TODO(elsk): b/33754152 Should not wrap this if object is Bs*
@@ -416,12 +455,14 @@ static void wrapPassthroughArg(Formatter& out, const NamedReference<Type>* arg,
     }).sElse([&] {
         out << wrappedName << " = " << name << ";\n";
     }).endl().endl();
+
+    return wrappedName;
 }
 
-void AST::generatePassthroughMethod(Formatter& out, const Method* method) const {
+void AST::generatePassthroughMethod(Formatter& out, const Method* method, const Interface* superInterface) const {
     method->generateCppSignature(out);
 
-    out << " {\n";
+    out << " override {\n";
     out.indent();
 
     if (method->isHidlReserved()
@@ -442,20 +483,23 @@ void AST::generatePassthroughMethod(Formatter& out, const Method* method) const
     generateCppInstrumentationCall(
             out,
             InstrumentationEvent::PASSTHROUGH_ENTRY,
-            method);
-
+            method,
+            superInterface);
 
+    std::vector<std::string> wrappedArgNames;
     for (const auto &arg : method->args()) {
-        wrapPassthroughArg(out, arg, false /* addPrefixToName */, [&] {
+        std::string name = wrapPassthroughArg(out, arg, arg->name(), [&] {
             out << "return ::android::hardware::Status::fromExceptionCode(\n";
             out.indent(2, [&] {
                 out << "::android::hardware::Status::EX_TRANSACTION_FAILED,\n"
                     << "\"Cannot wrap passthrough interface.\");\n";
             });
         });
+
+        wrappedArgNames.push_back(name);
     }
 
-    out << "auto _hidl_error = ::android::hardware::Void();\n";
+    out << "::android::hardware::Status _hidl_error = ::android::hardware::Status::ok();\n";
     out << "auto _hidl_return = ";
 
     if (method->isOneway()) {
@@ -464,10 +508,8 @@ void AST::generatePassthroughMethod(Formatter& out, const Method* method) const
                ", mEnableInstrumentation = this->mEnableInstrumentation, "
                "mInstrumentationCallbacks = this->mInstrumentationCallbacks\n"
             << "#endif // __ANDROID_DEBUGGABLE__\n";
-        for (const auto &arg : method->args()) {
-            out << ", "
-                << (arg->type().isInterface() ? "_hidl_wrapped_" : "")
-                << arg->name();
+        for (const std::string& arg : wrappedArgNames) {
+            out << ", " << arg;
         }
         out << "] {\n";
         out.indent();
@@ -480,6 +522,15 @@ void AST::generatePassthroughMethod(Formatter& out, const Method* method) const
     out.join(method->args().begin(), method->args().end(), ", ", [&](const auto &arg) {
         out << (arg->type().isInterface() ? "_hidl_wrapped_" : "") << arg->name();
     });
+
+    std::function<void(void)> kHandlePassthroughError = [&] {
+        out << "_hidl_error = ::android::hardware::Status::fromExceptionCode(\n";
+        out.indent(2, [&] {
+            out << "::android::hardware::Status::EX_TRANSACTION_FAILED,\n"
+                << "\"Cannot wrap passthrough interface.\");\n";
+        });
+    };
+
     if (returnsValue && elidedReturn == nullptr) {
         // never true if oneway since oneway methods don't return values
 
@@ -497,46 +548,54 @@ void AST::generatePassthroughMethod(Formatter& out, const Method* method) const
         generateCppInstrumentationCall(
                 out,
                 InstrumentationEvent::PASSTHROUGH_EXIT,
-                method);
+                method,
+                superInterface);
 
+        std::vector<std::string> wrappedOutNames;
         for (const auto &arg : method->results()) {
-            wrapPassthroughArg(out, arg, true /* addPrefixToName */, [&] {
-                out << "_hidl_error = ::android::hardware::Status::fromExceptionCode(\n";
-                out.indent(2, [&] {
-                    out << "::android::hardware::Status::EX_TRANSACTION_FAILED,\n"
-                        << "\"Cannot wrap passthrough interface.\");\n";
-                });
-                out << "return;\n";
-            });
+            wrappedOutNames.push_back(
+                wrapPassthroughArg(out, arg, "_hidl_out_" + arg->name(), kHandlePassthroughError));
         }
 
         out << "_hidl_cb(";
-        out.join(method->results().begin(), method->results().end(), ", ", [&](const auto &arg) {
-            out << (arg->type().isInterface() ? "_hidl_out_wrapped_" : "_hidl_out_")
-                << arg->name();
-        });
+        out.join(wrappedOutNames.begin(), wrappedOutNames.end(), ", ",
+                 [&](const std::string& arg) { out << arg; });
         out << ");\n";
         out.unindent();
         out << "});\n\n";
     } else {
         out << ");\n\n";
 
-        // used by generateCppInstrumentationCall
         if (elidedReturn != nullptr) {
-            out << "#ifdef __ANDROID_DEBUGGABLE__\n"
-                << elidedReturn->type().getCppResultType() << " _hidl_out_" << elidedReturn->name()
-                << " = _hidl_return;\n"
-                << "#endif // __ANDROID_DEBUGGABLE__\n";
+            const std::string outName = "_hidl_out_" + elidedReturn->name();
+
+            out << elidedReturn->type().getCppResultType() << " " << outName
+                << " = _hidl_return;\n";
+            out << "(void) " << outName << ";\n";
+
+            const std::string wrappedName =
+                wrapPassthroughArg(out, elidedReturn, outName, kHandlePassthroughError);
+
+            if (outName != wrappedName) {
+                // update the original value since it is used by generateCppInstrumentationCall
+                out << outName << " = " << wrappedName << ";\n\n";
+
+                // update the value to be returned
+                out << "_hidl_return = " << outName << "\n;";
+            }
         }
         generateCppInstrumentationCall(
                 out,
                 InstrumentationEvent::PASSTHROUGH_EXIT,
-                method);
+                method,
+                superInterface);
     }
 
     if (method->isOneway()) {
         out.unindent();
         out << "});\n";
+    } else {
+        out << "if (!_hidl_error.isOk()) return _hidl_error;\n";
     }
 
     out << "return _hidl_return;\n";
@@ -573,6 +632,7 @@ void AST::generateMethods(Formatter& out, const MethodGenerator& gen, bool inclu
 }
 
 void AST::generateTemplatizationLink(Formatter& out) const {
+    DocComment("The pure class is what this class wraps.").emit(out);
     out << "typedef " << mRootScope.getInterface()->localName() << " Pure;\n\n";
 }
 
@@ -633,6 +693,8 @@ void AST::generateStubHeader(Formatter& out) const {
 
     out.endl();
     generateTemplatizationLink(out);
+    DocComment("Type tag for use in template logic that indicates this is a 'native' class.")
+            .emit(out);
     generateCppTag(out, "android::hardware::details::bnhw_tag");
 
     out << "::android::sp<" << iface->localName() << "> getImpl() { return _hidl_mImpl; }\n";
@@ -725,6 +787,8 @@ void AST::generateProxyHeader(Formatter& out) const {
         << "\n\n";
 
     generateTemplatizationLink(out);
+    DocComment("Type tag for use in template logic that indicates this is a 'proxy' class.")
+            .emit(out);
     generateCppTag(out, "android::hardware::details::bphw_tag");
 
     out << "virtual bool isRemote() const override { return true; }\n\n";
@@ -779,9 +843,12 @@ void AST::generateCppSource(Formatter& out) const {
         << mPackage.string() << "::" << baseName
         << "\"\n\n";
 
-    out << "#include <android/log.h>\n";
+    out << "#include <log/log.h>\n";
     out << "#include <cutils/trace.h>\n";
     out << "#include <hidl/HidlTransportSupport.h>\n\n";
+    out << "#include <hidl/Static.h>\n";
+    out << "#include <hwbinder/ProcessState.h>\n";
+    out << "#include <utils/Trace.h>\n";
     if (iface) {
         // This is a no-op for IServiceManager itself.
         out << "#include <android/hidl/manager/1.0/IServiceManager.h>\n";
@@ -998,7 +1065,7 @@ void AST::generateProxyMethodSource(Formatter& out, const std::string& klassName
 }
 
 void AST::generateStaticProxyMethodSource(Formatter& out, const std::string& klassName,
-                                          const Method* method) const {
+                                          const Method* method, const Interface* superInterface) const {
     if (method->isHidlReserved() && method->overridesCppImpl(IMPL_PROXY)) {
         return;
     }
@@ -1037,7 +1104,8 @@ void AST::generateStaticProxyMethodSource(Formatter& out, const std::string& kla
     generateCppInstrumentationCall(
             out,
             InstrumentationEvent::CLIENT_API_ENTRY,
-            method);
+            method,
+            superInterface);
 
     out << "::android::hardware::Parcel _hidl_data;\n";
     out << "::android::hardware::Parcel _hidl_reply;\n";
@@ -1091,7 +1159,7 @@ void AST::generateStaticProxyMethodSource(Formatter& out, const std::string& kla
         << " */, _hidl_data, &_hidl_reply";
 
     if (method->isOneway()) {
-        out << ", " << Interface::FLAG_ONEWAY << " /* oneway */";
+        out << ", " << Interface::FLAG_ONE_WAY->cppValue();
     }
     out << ");\n";
 
@@ -1144,7 +1212,8 @@ void AST::generateStaticProxyMethodSource(Formatter& out, const std::string& kla
     generateCppInstrumentationCall(
             out,
             InstrumentationEvent::CLIENT_API_EXIT,
-            method);
+            method,
+            superInterface);
 
     if (elidedReturn != nullptr) {
         out << "_hidl_status.setFromStatusT(_hidl_err);\n";
@@ -1198,8 +1267,8 @@ void AST::generateProxySource(Formatter& out, const FQName& fqName) const {
     out << "}\n\n";
 
     generateMethods(out,
-                    [&](const Method* method, const Interface*) {
-                        generateStaticProxyMethodSource(out, klassName, method);
+                    [&](const Method* method, const Interface* superInterface) {
+                        generateStaticProxyMethodSource(out, klassName, method, superInterface);
                     },
                     false /* include parents */);
 
@@ -1234,11 +1303,12 @@ void AST::generateStubSource(Formatter& out, const Interface* iface) const {
         << "\") { \n";
     out.indent();
     out << "_hidl_mImpl = _hidl_impl;\n";
-    out << "auto prio = ::android::hardware::details::gServicePrioMap.get("
+    out << "auto prio = ::android::hardware::details::gServicePrioMap->get("
         << "_hidl_impl, {SCHED_NORMAL, 0});\n";
     out << "mSchedPolicy = prio.sched_policy;\n";
     out << "mSchedPriority = prio.prio;\n";
-    out << "setRequestingSid(::android::hardware::details::gServiceSidMap.get(_hidl_impl, false));\n";
+    out << "setRequestingSid(::android::hardware::details::gServiceSidMap->get(_hidl_impl, "
+           "false));\n";
     out.unindent();
 
     out.unindent();
@@ -1271,12 +1341,14 @@ void AST::generateStubSource(Formatter& out, const Interface* iface) const {
 
     out << klassName << "::~" << klassName << "() ";
     out.block([&]() {
-        out << "::android::hardware::details::gBnMap.eraseIfEqual(_hidl_mImpl.get(), this);\n";
-    }).endl().endl();
+           out << "::android::hardware::details::gBnMap->eraseIfEqual(_hidl_mImpl.get(), this);\n";
+       })
+            .endl()
+            .endl();
 
     generateMethods(out,
-                    [&](const Method* method, const Interface*) {
-                        return generateStaticStubMethodSource(out, iface->fqName(), method);
+                    [&](const Method* method, const Interface* superInterface) {
+                        return generateStaticStubMethodSource(out, iface->fqName(), method, superInterface);
                     },
                     false /* include parents */);
 
@@ -1323,8 +1395,8 @@ void AST::generateStubSource(Formatter& out, const Interface* iface) const {
 
         out.indent();
 
-        out << "bool _hidl_is_oneway = _hidl_flags & " << Interface::FLAG_ONEWAY
-            << " /* oneway */;\n";
+        out << "bool _hidl_is_oneway = _hidl_flags & " << Interface::FLAG_ONE_WAY->cppValue()
+            << ";\n";
         out << "if (_hidl_is_oneway != " << (method->isOneway() ? "true" : "false") << ") ";
         out.block([&] { out << "return ::android::UNKNOWN_ERROR;\n"; }).endl().endl();
 
@@ -1394,7 +1466,7 @@ void AST::generateStubSourceForMethod(Formatter& out, const Method* method,
 }
 
 void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
-                                         const Method* method) const {
+                                         const Method* method, const Interface* superInterface) const {
     if (method->isHidlReserved() && method->overridesCppImpl(IMPL_STUB)) {
         return;
     }
@@ -1459,7 +1531,8 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
     generateCppInstrumentationCall(
             out,
             InstrumentationEvent::SERVER_API_ENTRY,
-            method);
+            method,
+            superInterface);
 
     const bool returnsValue = !method->results().empty();
     const NamedReference<Type>* elidedReturn = method->canElideCallback();
@@ -1488,6 +1561,7 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
         });
 
         out << ");\n\n";
+
         out << "::android::hardware::writeToParcel(::android::hardware::Status::ok(), "
             << "_hidl_reply);\n\n";
 
@@ -1511,7 +1585,8 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
         generateCppInstrumentationCall(
                 out,
                 InstrumentationEvent::SERVER_API_EXIT,
-                method);
+                method,
+            superInterface);
 
         out << "_hidl_cb(*_hidl_reply);\n";
     } else {
@@ -1519,7 +1594,8 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
             out << "bool _hidl_callbackCalled = false;\n\n";
         }
 
-        out << callee << "->" << method->name() << "(";
+        out << "::android::hardware::Return<void> _hidl_ret = " << callee << "->" << method->name()
+            << "(";
 
         out.join(method->args().begin(), method->args().end(), ", ", [&] (const auto &arg) {
             if (arg->type().resultNeedsDeref()) {
@@ -1581,7 +1657,8 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
             generateCppInstrumentationCall(
                     out,
                     InstrumentationEvent::SERVER_API_EXIT,
-                    method);
+                    method,
+                    superInterface);
 
             out << "_hidl_cb(*_hidl_reply);\n";
 
@@ -1593,9 +1670,12 @@ void AST::generateStaticStubMethodSource(Formatter& out, const FQName& fqName,
             generateCppInstrumentationCall(
                     out,
                     InstrumentationEvent::SERVER_API_EXIT,
-                    method);
+                    method,
+                    superInterface);
         }
 
+        out << "_hidl_ret.assertOk();\n";
+
         if (returnsValue) {
             out << "if (!_hidl_callbackCalled) {\n";
             out.indent();
@@ -1669,8 +1749,8 @@ void AST::generatePassthroughHeader(Formatter& out) const {
     generateTemplatizationLink(out);
     generateCppTag(out, "android::hardware::details::bs_tag");
 
-    generateMethods(out, [&](const Method* method, const Interface*) {
-        generatePassthroughMethod(out, method);
+    generateMethods(out, [&](const Method* method, const Interface* superInterface) {
+        generatePassthroughMethod(out, method, superInterface);
     });
 
     out.unindent();
@@ -1721,7 +1801,7 @@ void AST::generateInterfaceSource(Formatter& out) const {
     });
 
     for (const Interface *superType : iface->typeChain()) {
-        out << "// static \n::android::hardware::Return<"
+        out << "::android::hardware::Return<"
             << childTypeResult
             << "> "
             << iface->localName()
@@ -1813,12 +1893,6 @@ void AST::generateCppAtraceCall(Formatter &out,
                 << baseString + "::server\");\n";
             break;
         }
-        case CLIENT_API_ENTRY:
-        {
-            out << "atrace_begin(ATRACE_TAG_HAL, \""
-                << baseString + "::client\");\n";
-            break;
-        }
         case PASSTHROUGH_ENTRY:
         {
             out << "atrace_begin(ATRACE_TAG_HAL, \""
@@ -1826,12 +1900,21 @@ void AST::generateCppAtraceCall(Formatter &out,
             break;
         }
         case SERVER_API_EXIT:
-        case CLIENT_API_EXIT:
         case PASSTHROUGH_EXIT:
         {
             out << "atrace_end(ATRACE_TAG_HAL);\n";
             break;
         }
+        // client uses scope because of gotos
+        // this isn't done for server because the profiled code isn't alone in its scope
+        // this isn't done for passthrough becuase the profiled boundary isn't even in the same code
+        case CLIENT_API_ENTRY: {
+            out << "::android::ScopedTrace PASTE(___tracer, __LINE__) (ATRACE_TAG_HAL, \""
+                << baseString + "::client\");\n";
+            break;
+        }
+        case CLIENT_API_EXIT:
+            break;
         default:
         {
             CHECK(false) << "Unsupported instrumentation event: " << event;
@@ -1842,7 +1925,8 @@ void AST::generateCppAtraceCall(Formatter &out,
 void AST::generateCppInstrumentationCall(
         Formatter &out,
         InstrumentationEvent event,
-        const Method *method) const {
+        const Method *method,
+        const Interface* superInterface) const {
     generateCppAtraceCall(out, event, method);
 
     out << "#ifdef __ANDROID_DEBUGGABLE__\n";
@@ -1920,18 +2004,16 @@ void AST::generateCppInstrumentationCall(
         }
     }
 
-    const Interface* iface = mRootScope.getInterface();
-
     out << "for (const auto &callback: mInstrumentationCallbacks) {\n";
     out.indent();
     out << "callback("
         << event_str
         << ", \""
-        << mPackage.package()
+        << superInterface->fqName().package()
         << "\", \""
-        << mPackage.version()
+        << superInterface->fqName().version()
         << "\", \""
-        << iface->localName()
+        << superInterface->localName()
         << "\", \""
         << method->name()
         << "\", &_hidl_args);\n";
