@@ -21,9 +21,14 @@ function run() {
     )
 
     $ANDROID_BUILD_TOP/build/soong/soong_ui.bash --make-mode -j \
-        ${RUN_TIME_TESTS[*]} ${SCRIPT_TESTS[*]} || return
+        ${RUN_TIME_TESTS[*]} ${SCRIPT_TESTS[*]} || exit 1
 
-    adb sync || return
+    # TODO(b/129507417): build with supported configurations
+    mkdir -p $ANDROID_PRODUCT_OUT/data/framework/hidl_test_java.jar || exit 1
+    cp $ANDROID_PRODUCT_OUT/testcases/hidl_test_java_java/arm64/hidl_test_java_java.jar \
+       $ANDROID_PRODUCT_OUT/data/framework/hidl_test_java_java.jar || exit 1
+
+    adb sync data || exit 1
 
     local BITNESS=("nativetest" "nativetest64")
 
@@ -48,6 +53,7 @@ function run() {
         for failed in ${FAILED_TESTS[@]}; do
             echo "FAILED TEST: $failed"
         done
+        exit 1
     else
         echo "SUCCESS"
     fi
