@@ -8,11 +8,13 @@ function run() {
         hidl_export_test \
         hidl_hash_test \
         hidl_impl_test \
+        hidl_system_api_test \
         android.hardware.tests.foo@1.0-vts.driver \
         android.hardware.tests.foo@1.0-vts.profiler)
 
     local RUN_TIME_TESTS=(\
         libhidl-gen-utils_test \
+        libhidl-gen-host-utils_test \
         hidl-gen-host_test \
     )
 
@@ -41,4 +43,4 @@ function run() {
     fi
 }
 
-run
\ No newline at end of file
+run
